/* 
    Adam Sosnowski - J1TIZ
    A.W.Sosnowski@stud.elka.pw.edu.pl
    Temat: biblioteka stl, opis algorytmu : partial_sum
    File: partial_sum.cpp
*/

/*
    ------- partial_sum -------
    
    Zadaniem algorytmu jest liczenie sum czesciowych.
    Innymi slowy liczy on sume wszystkich elementow poprzedzajacych
    dana pozycje w ciagu na ktorym zostanie uzyty i wstawia je do ciagu 'result'.
    Wystepuje on rowniez w drugiej 'wersji', gdzie sumowanie elementow zastepuje sie inna,
    wybrana przez programiste operacja.    
*/

/*
    ------- Parametry -------
    
    ----- 'wersja' podstawowa -----
    template <class InputIterator, class OutputIterator>
       OutputIterator partial_sum ( InputIterator first, InputIterator last,
                                    OutputIterator result );
    
    ----- 'wersja' pozwalajaca na uzycie algorytmu z operacja inna niz sumowanie -----
    template <class InputIterator, class OutputIterator, class BinaryOperation>
       OutputIterator partial_sum ( InputIterator first, InputIterator last,
                                    OutputIterator result, BinaryOperation binary_op );
    
    ----- opis parametrow -----                                
    first, last - iteratory do pierwszego i ostatniego elementu ciagu. Przedzial [first, last].
    result -  iterator do ciagu z wynikami dzialania partial_sum
    binary_op - operacja (obiekt lub funkcja) przyjmujaca za argumenty elementy wskazywane przez iteratory z 
              przedzialu [first, last] i zwracajaca wynik swojego dzialania (zamiast sumy).
*/

/*
    ------- Wartosc zwracana -------
    
    Iterator na koniec stworzonego ciagu.
*/

/* 
    ------- Implementacja algorytmu partial_sum -------
   
    template <class InputIterator, class OutputIterator>
    OutputIterator partial_sum ( InputIterator first, InputIterator last,
                                 OutputIterator result )
    {
      iterator_traits<InputIterator>::value_type val;
      *result++ = val = *first++;
      while (first!=last)
        *result++ = val = val + *first++;
               // or: *result++ = val = binary_op(val,*first++) for binary_op version
      return result;
    }
*/

/*
    ------- Zlozonosc -------
    Zlozonosc algorytmu ma charakter liniowy
    Dokladnie jest to: (last - first) - 1 + zlozonosc binary_op (jesli jest uzywany)
*/


/*
    ------- Przyklady -------
    Prezentacja uzycia algorytmu partial_sum na wektorach i listach.
*/


#include <iostream>
#include <vector>
#include <functional>
#include <list>

/* Tu znajduje sie m.in partial_sum */
#include <numeric>

using namespace std;


/* Wlasna, prosta funkcja na potrzeby przykladu */
int moja_funkcja(const int& a, const int& b)
{
    return (a+b+50);
}



int main()
{
   
   vector<int> vec_1(10), vec_2(10);
   vector<int>::iterator vec_iter_1, vec_iter_2, v_iter_end_1, v_iter_end_2;
   
   list <int> li_1;
   list <int>::iterator li_iter, li_iter_end;


   /* Inicjowanie listy kolejnymi liczbami naturalnymi */
   for (int i = 1; i <= 10; ++i)
   {
      li_1.push_back(i);
   }

   /* Wypisywanie zawartosci listy */
   cout << "Wartosci kolejnych elementow listy:\n [ " ;
   for (li_iter = li_1.begin(); li_iter != li_1.end(); ++li_iter)
   {
      cout << *li_iter << " ";
   }
   cout << "]" << endl;
   
   
   /* Uzycie pierwszej 'wersji' algorytmu partial_sum.
      Elementy z listy sumowane sa do wektora */
   v_iter_end_1 = partial_sum (li_1.begin (), li_1.end (), vec_1.begin());
   
   cout << "Zawartosc wektora (ciagu) po uzyciu algorytmu partial_sum:\n [ " ;
   for ( vec_iter_1 = vec_1.begin(); vec_iter_1 != v_iter_end_1; ++vec_iter_1)
   {
      cout << *vec_iter_1 << " ";
   }
   cout << "]" << endl;

   /* Uzycie drugiej 'wersji' algorytmu partial_sum.
      Uzywamy funkcji multiplies.
      Dane wejsciowe: elementy z listy.
      Dane wyjsciowe: wektor. */
   v_iter_end_2 = partial_sum (li_1.begin(), li_1.end(), vec_2.begin(), multiplies<int>());
   
   cout << "Zawartosc wektora (ciagu) po uzyciu algorytmu partial_sum w drugiej 'wersji'\n"
            << "(Uzyto funkcji multiplies): \n [ " ;
   for ( vec_iter_2 = vec_2.begin(); vec_iter_2 != v_iter_end_2 ; ++vec_iter_2)
   {
      cout << *vec_iter_2 << " ";
   }
   cout << "]" << endl;

   /* Uzycie drugiej 'wersji' algorytmu partial_sum.
      Uzywamy wlasnej funkcji.
      Dane wejsciowe: elementy z listy.
      Dane wyjsciowe: lista. */
   li_iter_end = partial_sum (li_1.begin(), li_1.end (), li_1.begin(), moja_funkcja);
   cout << "Zawartosc listy (ciagu) po uzyciu algorytmu partial_sum w drugiej 'wersji'\n"
            << "(Uzyto wlasnej funkcji): \n [ " ;
   for (li_iter = li_1.begin(); li_iter != li_iter_end; ++li_iter)
   {
      cout << *li_iter << " ";
   }
   cout << "]" << endl;
   
   
   return 0;
   
}




